/**
 * A Clock model implementation that keeps time but no date
 **/
(function(window, $){
	
	/**
	 * Represent a Clock that keeps time but no date
	 * @class Clock
	 **/
	var Clock = function(hours, minutes, seconds){
		this.initialize(hours, minutes, seconds);
	}
	
	Clock.prototype = {
		//Hours
		get hours(){
			var value = Math.floor(_totalTimeInSeconds / 3600)%24;
			while(value < 0){
				value += 24;
			}
			return value; 
		},
		set hours(value){
			var curHours = this.hours;
			
			if(curHours == value)
				return;
			
			var diff = value - curHours;
			this.totalTimeInSeconds += diff*3600;
		},
		//Minutes
		get minutes(){
			var value = Math.floor((_totalTimeInSeconds%3600)/60);
			while(value < 0)
				value += 60;
			return value;
		},
		set minutes(value){
			var curMinutes = this.minutes;
			
			if(curMinutes == value)
				return;
			
			var diff = value - curMinutes;
			this.totalTimeInSeconds += diff*60;
		},
		//Seconds
		get seconds(){
			var value = this.totalTimeInSeconds%60;
			while(value < 0)
				value += 60;
			return value;
		},
		set seconds(value){
			var curSeconds = this.seconds;
			
			if(value == curSeconds)
				return;
			
			var diff = value - curSeconds;
			this.totalTimeInSeconds += diff;		
		},
		get totalTimeInSeconds(){
			return _totalTimeInSeconds;
		},
		set totalTimeInSeconds(value){
			if(_totalTimeInSeconds == value)
				return;			
				
			_totalTimeInSeconds = value;
			
			this.sendUpdate();
		}
	};
	
	var p = Clock.prototype;

//Private properties

	//Note: A single unit of seconds it kept to store the value of the clock. This makes a lot of code cleaner than keeping
	//separate values for hours, minutes etc.
	var _totalTimeInSeconds;

// Public properties
	
//Constructor

	/**
	 * Initialization method
	**/
	p.initialize = function(hours, minutes, seconds){
		_totalTimeInSeconds = hours*3600 + minutes*60 + seconds;
	}
	
//Public methods

	p.reset = function(){
		self.totalTimeInSeconds = 0;
	};

	p.sendUpdate = function(){
		$.publish('clock:update', this);
	};

	p.toString = function(){
		return window.StringUtils.pad(this.hours, 2)+":"+window.StringUtils.pad(this.minutes, 2)+":"+window.StringUtils.pad(Math.round(this.seconds),2);
	};

	window.Clock = Clock;
})(window, jQuery);