/**
 * An ClockView implementation
 **/
(function(window, $){
	
	/**
	 * Represent a View of an analog clock 
	 * @class Clock
	 **/
	var ClockView = function(clock, skin){
		this.initialize(clock, skin);
	}
	
	ClockView.prototype = {
		//Graphic of the clock
		get skin(){
			return _skin;
		}
	};
	
	var p = ClockView.prototype;

//Private properties
	var _clock = null;
	var _skin = null;

// Public properties	

	
//Public methods
	p.update = function(){
		_skin.hourHandle.rotation = 360 * (((_clock.hours + _clock.minutes/60)%12)/12);
		_skin.minuteHandle.rotation = 360*((_clock.minutes%60)/60);
	}

	p.onClockUpdated = function(event, clock){
		this.update();
	}
	
	p.setNeutral = function(){
		_skin.clockFace.gotoAndStop('neutral');
	};
	
	p.setCorrect = function(){
		_skin.clockFace.gotoAndStop('ok');
	};	
	
	p.setIncorrect = function(){
		_skin.clockFace.gotoAndStop('nok');
	};
	
//Constructor
	/**
	 * Initialization method
	**/
	p.initialize = function(clock, skin){
		_clock = clock;
		_skin = skin;
		
		//Setup interaction on the hourHandle
		_skin.hourHandle.button.onPress = function(e){
			//Switch image
			_skin.hourHandle.button.gotoAndStop('down');
			
			e.onMouseMove = function(e) {
				//Convert to local on the clock
				var local = _skin.globalToLocal(e.stageX, e.stageY);
				var x = local.x ;
				var y = local.y ;
				
				//The conversionfactor
				var angleFactor = 12*3600 / (Math.PI*2);		

				//Calculate the delta angle between the mouse and handle
				var curAngle = (((_clock.hours + _clock.minutes/60)%12)/12) * (Math.PI*2);							
				var targetAngle =  NumberUtils.coordinateAngle(x, y);	
				var angleDiff = NumberUtils.shortestAngle(targetAngle, curAngle);

				//Round it to whole hours
				var secondsDiff = Math.round(angleDiff*angleFactor);
				var hoursDiff = Math.round(secondsDiff / 3600);
				_clock.totalTimeInSeconds = Math.round(clock.totalTimeInSeconds + hoursDiff*3600);
			}

			e.onMouseUp = function(e) {
				//Switch back to original image
				_skin.hourHandle.button.gotoAndStop('up');
			}
		}
		
		//Setup interaction on the minuteHandle
		_skin.minuteHandle.button.onPress = function(e){
			//Switch image
			_skin.minuteHandle.button.gotoAndStop('down');

			e.onMouseMove = function(e) {
				//Convert to local on the clock
				var local = _skin.globalToLocal(e.stageX, e.stageY);
				var x = local.x;
				var y = local.y;
				
				//The conversionfactor
				var angleFactor = 3600 / (Math.PI*2);		

				//Calculate the delta angle between the mouse and handle
				var curAngle = _clock.minutes * 60 / angleFactor;
				var targetAngle =  NumberUtils.coordinateAngle(x, y);										
				var angleDiff = NumberUtils.shortestAngle(targetAngle, curAngle);
				
				//Adjust the clock
				_clock.totalTimeInSeconds = Math.round(clock.totalTimeInSeconds + angleDiff*angleFactor);
			}

			e.onMouseUp = function(e) {
				//Switch back to original image
				_skin.minuteHandle.button.gotoAndStop('up');
			}
		}
		
		if(_clock){
			//FIXME: should subscribe on the clock, not global events
			$.subscribe('clock:update', $.proxy(this.onClockUpdated, this));
		}
		
		//Update the clockview
		this.update();
	}
	window.ClockView = ClockView;	
})(window, jQuery);