/**
 * The ClockWidget 
 **/
(function(window, $){
	
	var ClockWidget = function(stage){
		this.initialize(stage);
	}
	
	ClockWidget.prototype = {
		//Graphic of the clock
		get skin(){
			return _skin;
		}
	};
	
	var p = ClockWidget.prototype;

//Private properties
	var _stage;
	var _skin;
	var _clock;
	var _clockView;
	var _digitalClockInput;
	
	//TMP
//	var _text;

// Public properties
	
//Constructor

	/**
	 * Initialization method
	**/
	p.initialize = function(canvas){
		_stage = new Stage(canvas);
		_stage.mouseEventsEnabled = true;
		Touch.enable(_stage);	
			
		//Initialize the model
		_clock = new Clock(0,0,0);
		
		//Initialize the view
		_skin = new lib.ClockWidgetSkin();
		
		_skin.message12hourDisplay.alpha = 0;
		_stage.addChild(_skin);
		
		//Stop the correct animation
		_skin.correctAnimation.gotoAndStop(0);
		
		//Initialize the analog clock view mediator
		_clockView = new ClockView(_clock, _skin.analogClock);
		
		//Initialize the digital clock view mediator
		_digitalClockInput = new DigitalClockInput(_skin.digitalClock, $("#hourInput"), $("#minuteInput"));		

		//Initial update of the stage
		_stage.update();

		//Set the framerate
		Ticker.setFPS(12);
		//Start listening to ticks
		Ticker.addListener(this);
		
		//TMP add text for debugging
/*		_text = new Text("","24px bold Arial", "#000000" );
		_text.x = 10;
		_text.y = 50;
		_stage.addChild(_text);
*/		
		//Load state
		this.load();

		//Listen to changes of the clock
		if(_clock){
			//FIXME: should subscribe on the clock, not global events
			$.subscribe('clock:update', $.proxy(this.onClockUpdated, this));
		}
		
		//Subscribe to an input event
		$.subscribe('input:started', $.proxy(this.onInputStarted, this));
		
		//Events for 12hour message
		$.subscribe('message12hourdisplay', $.proxy(this.show12hourdisplayMessage, this));
		
		//Add some hooks for saving
		if(isDefined(NB)){
			NB.addObserver("widgetWillSuspend", $.proxy(this.save, this));
			NB.addObserver("pageLeaveEvent", $.proxy(this.save, this));
			
			//NOTE: there is no event for saving the file or closing the document
			// so we setup an autosave this way
			setInterval($.proxy(this.save, this), 2000);
		}
	}
	
//Public methods

	var _startupInstructionsHidden = false;

	p.hideStartupInstructions = function(){
		this.hide12hourdisplayMessage();
		
		if(_startupInstructionsHidden)
			return;
	
		Tween.get(_skin.startupInstructions).to({alpha:0}, 1000);
		
		_startupInstructionsHidden = true;	
	};

	p.showStartupInstructions = function(){
		if(!_startupInstructionsHidden)
			return;
	
		Tween.get(_skin.startupInstructions).to({alpha:1}, 1000);	
		_startupInstructionsHidden = false;	
	};
	
	p.show12hourdisplayMessage = function(){
		Tween.get(_skin.message12hourDisplay).to({alpha:1}, 1000);
	}
	
	p.hide12hourdisplayMessage = function(){
		Tween.get(_skin.message12hourDisplay).to({alpha:0}, 1000);
	}
	
	p.tick = function(){
		_stage.update();

		//Check for correct match
		if((_digitalClockInput.hours % 12) == (_clock.hours % 12) && _digitalClockInput.minutes == _clock.minutes){
			this.setCorrect();
		} else {
			this.setIncorrect();		
		}
	};
	
	p.getAnalogClockText = function(){
		return ""+StringUtils.pad(_clock.hours % 12, 2)+":"+StringUtils.pad(_clock.minutes, 2);				
	}
	
	p.getDigitalClockText = function(){
		return ""+StringUtils.pad(_digitalClockInput.hours % 24, 2)+":"+StringUtils.pad(_digitalClockInput.minutes, 2);				
	}
	
	var _correctAnimationDisplayed = false;
	
	p.setCorrect = function(){
		//Play animation		
		if(!_correctAnimationDisplayed){
			_skin.correctAnimation.gotoAndPlay(0);
			_correctAnimationDisplayed = true;
		}
		
		_clockView.setCorrect();
		_digitalClockInput.setCorrect();
	};
	
	p.setIncorrect = function(){
		//Reset animation
		_correctAnimationDisplayed = false;
		
		_clockView.setNeutral();
		
		//If the digital clock input is showing incorrect input we do not set neutral
		if(!_digitalClockInput.isIncorrect())
			_digitalClockInput.setNeutral();
	};
	
	p.reset = function(){
		_clock.totalTimeInSeconds = 0;
		_digitalClockInput.reset();
	};
	
	p.close = function(){
		//Remove ourself from Notebook
		NB.getHostObject().deleteObject();		
	};
	
	p.save = function(){
		//TMP
	//	return;
		
		//Save the totalTimeInSeconds
		NB.persist.save("totalTimeInSeconds", _clock.totalTimeInSeconds);
		NB.persist.save("digitalHours", _digitalClockInput.hours);
		NB.persist.save("digitalMinutes", _digitalClockInput.minutes);	
		NB.persist.save("accept24hInput", _digitalClockInput.accept24hInput);
	};
	
	p.load = function(){
		var value = NB.persist.load("totalTimeInSeconds");
		if(isDefined(value)){
			_clock.totalTimeInSeconds = parseInt(value);
		}
		
		value = NB.persist.load("digitalHours");
		if(isDefined(value)){
			_digitalClockInput.hours = parseInt(value);
		}

		value = NB.persist.load("digitalMinutes");
		if(isDefined(value)){
			_digitalClockInput.minutes = parseInt(value);
		}
		
		value = NB.persist.load("accept24hInput");	
		if(isDefined(value)){
			_digitalClockInput.accept24hInput = StringUtils.stringToBoolean(value);	
		}
		
	};
	
	p.openSettings = function(){
		console.log("opening settings");
		
		//Set the settings values
		if(_digitalClockInput.accept24hInput){
			$('input:radio[name=digitalClockFormat]')[1].checked = true;		
		} else {
			$('input:radio[name=digitalClockFormat]')[0].checked = true;;
		}
		
		//Switch view
		$('#frontend').hide(0, function(){
			$('#backend').show(0);
		});		
	}
	
	p.closeSettings = function(){
		console.log("closing settings");
		
		//Read the settings values
		_digitalClockInput.accept24hInput = $('input:radio[name=digitalClockFormat]')[1].checked;
		
		if(_digitalClockInput.accept24hInput){
			this.hide12hourdisplayMessage();
		}
		
		
		//Save the settings
//		this.save();
		
		//Switch view
		$('#backend').hide(0, function(){
			$('#frontend').show(0);	
		});		
	}
	
	p.openInfo = function(){
		console.log("opening info");
		//Switch view
		$('#frontend').hide(0, function(){
			$('#info').show(0);
		});
		
	}
	
	p.closeInfo = function(){
		console.log("closing info");
		$('#info').hide(0, function(){
			$('#frontend').show(0);
		});
	}
	
	p.onClockUpdated = function(event, clock){
		this.hideStartupInstructions();
//		this.save();
	}	
	
	p.onInputStarted = function(event){
		this.hideStartupInstructions();
	}
	
	p.inject = function(){
		console.log("Injecting");	
		
		if(!isDefined(_digitalClockInput.hours) || !isDefined(_digitalClockInput.minutes))
			return;
		
		var host = NB.getHostObject();
		var center = new Point(host.x + host.width/2, host.y + host.height/2);		
		
		var text = this.getDigitalClockText();
		var prototype = NB.objectPrototype.text(text, {x: host.x + host.width + 15, 
																			y: center.y - 100, 
																			width: text.length*50,
																			height: 200,
																			transparency:0});
		prototype.font.size = 48;
		prototype.font.name = "Comic Sans MS"
		var object = NB.addObject(prototype);
		object.animation.start({ speed: "Normal", repeats: "None", type: "Fade In" });		
	}
	
	
	
	window.ClockWidget = ClockWidget;
})(window, jQuery);