/**
 * An DigitalClockInput implementation
 **/
(function(window, $){
	
	/**
	 * Represent a View of an analog clock 
	 * @class Clock
	 **/
	var DigitalClockInput = function(skin, hoursInput, minutesInput){
		this.initialize(skin, hoursInput, minutesInput);
	}
	
	DigitalClockInput.prototype = {
		//Graphic of the clock
		get skin(){
			return _skin;
		},
		get accept24hInput(){
			return _accept24hInput;
		},
		set accept24hInput(value){
			_accept24hInput = value;
		},
		get hours(){
			return _hoursValue;
		},
		set hours(value){
			_hoursValue = value;
			_hoursInput.text(StringUtils.pad(_hoursValue, 2));
		},
		get minutes(){
			return _minutesValue;
		},
		set minutes(value){
			_minutesValue = value;
			_minutesInput.text(StringUtils.pad(_minutesValue, 2)); 
		}
	};
	
	var p = DigitalClockInput.prototype;

//Private properties
	var _skin = null;
	var _hoursInput = null;
	var _minutesInput = null;
	var _hoursValue = undefined;
	var _minutesValue = undefined;
	var _accept24hInput = true;
	var _isIncorrect = false;

// Public properties	

	
//Public methods

	p.inputStarted = function(text, obj){
		//Send an event
		$.publish('input:started', this);		
	}

	p.hoursChanged = function(text, obj){
		console.log('hours changed: "'+text+'"');
		
/*		//Replace something
		text = text.replace(/\s/g, "");
		text = text.replace(';', ':');
		var parts = text.split(':');
		if(parts.length > 1){
			this.hoursChanged(parts[0], obj);
			this.minutesChanged(parts[1], obj);
			return;
		}
	
		//Process value
		_hoursInput.text(text); 
		_hoursValue = parseInt(text, 10);*/
		
		_hoursValue = text;
		
		if(isNaN(_hoursValue) || _hoursValue < 0 || _hoursValue > 23){
			this.setIncorrect();
		} else {
			if(!_accept24hInput && _hoursValue > 12){
				//TODO: show message
				this.setIncorrect();
				//Show message
				$.publish('message12hourdisplay', this);
			} else {
				//Set 2 digit number
				_hoursInput.text(StringUtils.pad(_hoursValue, 2));			
			}
		}
		//Send an event
		$.publish('clock:update', this);
	};
	
	p.hoursDrop = function(text, obj){
		//Replace something
		text = text.replace(/\s/g, "");
		text = text.replace(';', ':');
		var parts = text.split(':');
		if(parts.length > 1){
			this.hoursChanged(parseInt(parts[0], 10), obj);
			this.minutesChanged(parseInt(parts[1],10), obj);
			return;
		}
	
		this.hoursChanged(parseInt(text, 10));
	}
	
	p.minutesChanged = function(text, obj) {
		console.log('minutes changed: "'+text+'"');
		

		
		_minutesValue = text;
		
		if(isNaN(_minutesValue) || _minutesValue < 0 || _minutesValue > 60){
			this.setIncorrect();
		} else {
			//Set 2 digit number
			_minutesInput.text(StringUtils.pad(_minutesValue, 2)); 
		}
		
		//Send an event
		$.publish('clock:update', this);
	};
	
	p.minutesDrop = function(text, obj){
		//Replace something
		text = text.replace(/\s/g, "");
		text = text.replace(';', ':');
		var parts = text.split(':');
		if(parts.length > 1){
			this.hoursChanged(parseInt(parts[0], 10), obj);
			this.minutesChanged(parseInt(parts[1],10), obj);
			return;
		}
	
		this.minutesChanged(parseInt(text, 10));
	}

	p.onInputChanged = function(){
		this.setIncorrect();
	};
	
	p.setNeutral = function(){
		_isIncorrect = false;
		_skin.gotoAndStop('neutral');
		
	};
	
	p.setCorrect = function(){
		_isIncorrect = false;
		_skin.gotoAndStop('ok');
	};
	
	p.setIncorrect = function(){
		_skin.gotoAndStop('nok');
		
		_isIncorrect = true;
		
		//Go back to default in a while
		setTimeout($.proxy(this.reset, this), 2000);
	};
	
	p.isIncorrect = function(){
		return _isIncorrect;
	}	
	
	p.reset = function(){
		_hoursValue = undefined;
		_minutesValue = undefined;
		_hoursInput.text('');
		_minutesInput.text('');		
		this.setNeutral();
	}
	
//Constructor
	/**
	 * Initialization method
	**/
	p.initialize = function(skin, hoursInput, minutesInput){
		_skin = skin;
		_hoursInput = hoursInput;
		_minutesInput = minutesInput;
		
		//Add text element to the display list
		var hoursField = new DOMElement(hoursInput.get(0));
		_skin.addChild(hoursField);
		
		//Add text element to the display list
		var minutesField = new DOMElement(minutesInput.get(0));
		minutesField.x = 113;
		
		_skin.addChild(minutesField);
		
		//Create an handwrite input field for hours
		NB.helpers.handwritable(_hoursInput, $.proxy(this.hoursChanged, this), { 
			timeToRecognize: 2, 
			wantsNumbers: true, 
			activeText: "..",
			deleteWhenDone: true,
			onRecognizing: $.proxy(this.inputStarted, this)	
		});
		NB.helpers.dragAndDropTextfield(_hoursInput, $.proxy(this.hoursDrop, this),{
			wantsNumbers: false, 
			deleteWhenDone: true	
		});
		
		//Create an handwrite input field for minutes
		NB.helpers.handwritable(_minutesInput, $.proxy(this.minutesChanged, this), { 
			timeToRecognize: 2, 
			wantsNumbers: true, 
			activeText: "..",
			deleteWhenDone: true,
			onRecognizing: $.proxy(this.inputStarted, this)
		});
		NB.helpers.dragAndDropTextfield(_minutesInput, $.proxy(this.minutesDrop, this),{
			wantsNumbers: false, 
			deleteWhenDone: true	
		});
		

		//Reset state
		this.reset();
	}
	window.DigitalClockInput = DigitalClockInput;	
})(window, jQuery);