/**
 * A simple library file with common js code, will have to be reorganized later on
**/

(function(window){
	/**
	 * A static class with string utilities
	 **/
	var StringUtils = function(){
		throw 'StringUtils cannot be initialized';
	}
	/**
	 * Add leading zero's to a number to get a string of a certain length
	 **/
	StringUtils.pad = function(number, length){
	    var str = '' + number;
	    while (str.length < length) {
	        str = '0' + str;
	    }
	    return str;		
	}
	
	StringUtils.stringToBoolean = function(string){
	        switch(string.toLowerCase()){
	                case "true": case "yes": case "1": return true;
	                case "false": case "no": case "0": case null: return false;
	                default: return Boolean(string);
	        }
	}
	
	window.StringUtils = StringUtils;
})(window);

(function(window){
	
	/**
	 * A static class with Number utilities
	 **/
	var NumberUtils = function(){
		throw 'NumberUtils cannot be initialized';
	}
	/**
	 * Returns the angle of the x,y coordinate in Radians between 0 - 2Pi
	 **/
	NumberUtils.coordinateAngle = function(x, y){
		var result = 0.5*Math.PI + Math.atan2(y,x);
		while(result < 0){
			result += 2*Math.PI;
		}
		while(result > 2*Math.PI){
			result -= 2*Math.PI;
		}
		return result;
	}	
	/**
	 * Returns the shortest angle between two angles. All in Radians
	 **/
	NumberUtils.shortestAngle = function(angle1, angle2){
		return Math.atan2(Math.sin(angle1 - angle2), Math.cos(angle1 - angle2));
	}
	/**
	 * Round a number to the specified number of decimals
	 **/
	NumberUtils.round = function(number, decimals){
		var newnumber = new Number(number+'').toFixed(parseInt(decimals));
		return parseFloat(newnumber); 
	}
	/**
	 * Clamp a number between a min and max
	**/
	NumberUtils.clamp = function(number, min, max){
		return Math.max(min, Math.min(max, number));
	}
	window.NumberUtils = NumberUtils;
})(window);

(function(window) {
	
	//If there is no notebook library setup we return
	if(!isDefined(NB) || !isDefined(NB.helpers)){
		return;
	}
	
	//Simple function that transforms a point to a local point on an element
	NB.helpers.transformPointToElement = function(pt, elem) {	
		var zoom = NB.page.getVisibleArea().width / 800.0;
		var hostObj = NB.getHostObject();
		var browserOffset = (hostObj.width - $(window).width()) / 2;

		//Transform x and y from NB space to browser document space
		var x = (pt.x - hostObj.x - browserOffset)/zoom;
		var y = (pt.y - hostObj.y - browserOffset)/zoom;	
		
		//Transform from document space to element space
		var elemTop = elem.offset().top / zoom; // get div top
		var elemLeft = elem.offset().left / zoom;
		x = (x - elemLeft) * zoom;
		y = (y - elemTop) * zoom;

		return new Point(x,y);
	};
	
	
	NB.helpers.pointInElement = function(pt, elem) {
		var zoom = NB.page.getVisibleArea().width / 800.0;
		var hostObj = NB.getHostObject();

		var browserOffset = (hostObj.width - $(window).width()) / 2;

		var elemTop = elem.offset().top / zoom; // get div top
		var elemLeft = elem.offset().left / zoom;
		var elemWidth = elem.width() / zoom;
		var elemHeight = elem.height() / zoom;

		var x = pt.x - hostObj.x - browserOffset;
		var y = pt.y - hostObj.y - browserOffset;
		x = x / zoom;
		y = y / zoom;

		DEBUG([x, y, elemLeft, elemTop, elemWidth, elemHeight]);
		return (x > elemLeft && x < (elemWidth + elemLeft) && y > elemTop && y < (elemHeight + elemTop));
	};	
	
	NB.helpers.normalizeToNumber = function (string) {
		var	numFromStringMap = { 
				0: ["o"],  
				1: ["i", "l"], 
				2: ["z"], 
				3: [], 
				4: ["A"], 
				5: [], 
				6: ["b"], 
				7: ["T"], 
				8: [], 
				9: ["g", "q"] 
		};
		
		// For each number look for its replacements and its uppercase equivilants and replace
		for(var num in numFromStringMap) {
			for(var index in numFromStringMap[num]) {
				string = string.replace(numFromStringMap[num][index], num);
				string = string.replace(numFromStringMap[num][index].toUpperCase(), num);
			}
		}

		return parseFloat(string);
	};
	
	NB.helpers.dragAndDropTextfield = function(elem, callback, options){
		var defaults = {
			deleteWhenDone: true,
			wantsNumbers: false
		};
		
		var options = $.extend({}, defaults, options);
		
		var dropObserver = NB.addObserver("dndDropEvent", function(evt){
			
			for(var guid in evt.eventData) {
				var contact =  evt.eventData[guid].contact;
				var pt = new Point(contact.x, contact.y);

				if(NB.helpers.pointInElement(pt, elem)){
					
					var obj = NB.getObject(guid);
					var returnValue = obj.getText();
					
					// If they wanted numebrs normalize
					if(options.wantsNumbers) {
						returnValue = NB.helpers.normalizeToNumber(returnValue);
					}

					// Call their callback function
					callback(returnValue, obj);
					
					// Delete when done
					if(options.deleteWhenDone) {
						// animate and remove
						obj.animation.start({ speed: "Fast", repeats: "None", type: "Fade Out" });
						obj.addObserver("annotationFinishedAnimatingEvent", function (evt) {
							var obj = NB.getObject(evt.eventData.guid);
							NB.page.deleteObject(obj);
						}); 
					}				
				} 
			}
		});		
	}		
})(window);



/*!
 * jQuery Tiny Pub/Sub - v0.3 - 11/4/2010
 * http://benalman.com/
 * 
 * Copyright (c) 2010 "Cowboy" Ben Alman
 * Dual licensed under the MIT and GPL licenses.
 * http://benalman.com/about/license/
 */
(function($){
  
  var o = $({});
  
  $.subscribe = function() {
    o.bind.apply( o, arguments );
  };
  
  $.unsubscribe = function() {
    o.unbind.apply( o, arguments );
  };
  
  $.publish = function() {
    o.trigger.apply( o, arguments );
  };
})(jQuery);