/**
 * The WeigingScaleWidget 
 **/
(function(window, $){
	
	var WeigingScaleWidget = function(stage){
		this.initialize(stage);
	}
	
	WeigingScaleWidget.prototype = {
		//Graphic of the clock
		get skin(){
			return _skin;
		},
		set currentWeightSetName(value){
			if(value == _currentWeightSetName)
				return;
			
			_currentWeightSetName = value;
			_currentWeightSet = weightSets[_currentWeightSetName];
						
			//Update some config elements
			this.makeAllWeightsVisible();
			this.updateWeightSelection();
			this.initWeightBarWeights();
			
			//Reset textfields		
			this.resetLeftTextInput();
			this.resetRightTextInput();
			
			//Reset everything
			this.reset();
		},
		get currentWeightSetName(){
			return _currentWeightSetName;
		}
	};
	
	var p = WeigingScaleWidget.prototype;

//Private properties
	//The main stage canvas
	var _stage;
	//The main sprite/container on the stage
	var _skin;
	//Input text fields
	var _textInputLeft;
	var _textInputRight;
	//Rotation settings y difference and rotation angle
	var _dy = 20;
	var _dr = 6;
	var _platformBaseY;
	
	//Sets of weights
	var _weightsBar = [];
	var _maxWeightsPerPlatform = 9;
	
	var _currentWeightSet;
	var _currentWeightSetName;
	
	//The number of weights in the skin
	var numWeights = 14;

// Public properties
	
//Constructor

	/**
	 * Initialization method
	**/
	p.initialize = function(canvas){
		_stage = new Stage(canvas);
		_stage.mouseEventsEnabled = true;
		Touch.enable(_stage);	
		
		//Initialize the view
		_skin = new lib.WeigingScaleSkin();
		_stage.addChild(_skin);
		
		//Stop the correct animation
		_skin.correctAnimation.gotoAndStop(0);

		//Add left text element to the display list
		var element = new DOMElement(document.getElementById("textInputLeft"));
		_skin.textInputLeft.addChild(element);
		_textInputLeft = $('#textInputLeft');
		NB.helpers.handwritable(_textInputLeft, $.proxy(this.leftTextInputChanged, this), { 
			timeToRecognize: 2, 
			wantsNumbers: false, 
			activeText: ".....",
			deleteWhenDone: true,
			onRecognizing: $.proxy(this.hideStartupInstructions, this)
		});
		NB.helpers.dragAndDropTextfield(_textInputLeft, $.proxy(this.leftTextInputChanged, this),{
			wantsNumbers: true, 
			deleteWhenDone: true	
		});
		

		//Add right text element to the display list
		var element = new DOMElement(document.getElementById("textInputRight"));
		_skin.textInputRight.addChild(element);
		_textInputRight = $('#textInputRight');
		NB.helpers.handwritable(_textInputRight, $.proxy(this.rightTextInputChanged, this), { 
			timeToRecognize: 2, 
			wantsNumbers: false, 
			activeText: ".....",
			deleteWhenDone: true,
			onRecognizing: $.proxy(this.hideStartupInstructions, this)
		});
		NB.helpers.dragAndDropTextfield(_textInputRight, $.proxy(this.rightTextInputChanged, this),{
			wantsNumbers: true, 
			deleteWhenDone: true	
		});
		
		//Store platform y to position scale later on 
		_platformBaseY = _skin.leftPlatform.y;
		
		//Create an empty container for weights on the left platform. Cannot be placed in the fla 
		// because empty containers are scripted out 
		_skin.leftPlatform.weights = new Container();
		_skin.leftPlatform.weights.x = _skin.leftPlatform.weightsContainerBackground.x;
		_skin.leftPlatform.weights.y = _skin.leftPlatform.weightsContainerBackground.y;
		_skin.leftPlatform.addChild(_skin.leftPlatform.weights);		
		_skin.rightPlatform.weights = new Container();
		_skin.rightPlatform.weights.x = _skin.rightPlatform.weightsContainerBackground.x;
		_skin.rightPlatform.weights.y = _skin.rightPlatform.weightsContainerBackground.y;
		_skin.rightPlatform.addChild(_skin.rightPlatform.weights);

		//Set the value
		$('#weightSetSelect').change($.proxy(function() {
		  this.currentWeightSetName = $('#weightSetSelect').val()
		}, this));
		this.currentWeightSetName = "grams";

		//Set the framerate
		Ticker.setFPS(12);
		//Start listening to ticks
		Ticker.addListener(this);
		
		//Reset state
		this.reset();
		
		//Load state
		this.load();
		
		//Initial update of the stage
		_stage.update();
		
		//Add some hooks for saving
		if(isDefined(NB)){
			NB.addObserver("widgetWillSuspend", $.proxy(this.save, this));
			NB.addObserver("pageLeaveEvent", $.proxy(this.save, this));
			
			//NOTE: there is no event for saving the file or closing the document
			// so we setup an autosave this way
			setInterval($.proxy(this.save, this), 2000);
		}
	}
	
//Public methods

	var _startupInstructionsHidden = false;

	p.hideStartupInstructions = function(){
		if(_startupInstructionsHidden)
			return;
	
		Tween.get(_skin.startupInstructions).to({alpha:0}, 1000);
		_startupInstructionsHidden = true;	
	};

	p.showStartupInstructions = function(){
		if(!_startupInstructionsHidden)
			return;
	
		Tween.get(_skin.startupInstructions).to({alpha:1}, 1000);	
		_startupInstructionsHidden = false;	
	};

	p.createNewWeight = function(value){
		var result = new lib.WeightSkin();
		result.gotoAndStop('normal');
		result.weightValue = value;
		result.scaleX = 0.33;
		result.scaleY = 0.33;
		result.weightTextField.text = this.formatWeightText(value);
		return result;
	}

	p.leftTextInputChanged = function (text, obj){
	    _textInputLeft.text(text);	

		if(isNaN(text)){
			// Do nothing
			_textInputLeft.text('NaN');
			setTimeout($.proxy(this.resetLeftTextInput, this), 1000);
		} else {
			if(_skin.leftPlatform.weights.getNumChildren() < _maxWeightsPerPlatform){
				var value = _currentWeightSet.round ? Math.round(text * _currentWeightSet.textInputMultiplyer) : text * _currentWeightSet.textInputMultiplyer;
				_textInputLeft.text(this.formatWeightText(value))
				this.addWeightLeft(this.createNewWeight(value));
				setTimeout($.proxy(this.resetLeftTextInput, this), 1500);
			}
		}
		this.hideStartupInstructions();
	};

	p.resetLeftTextInput = function(){
		_textInputLeft.text('? '+_currentWeightSet.textInputLabel);		
	}
	
	p.rightTextInputChanged = function (text, obj){
	    _textInputRight.text(text);	

		if(isNaN(text)){
			// Do nothing
			_textInputLeft.text('NaN');
			setTimeout($.proxy(this.resetRightTextInput, this), 1000);
		} else {
			if(_skin.rightPlatform.weights.getNumChildren() < _maxWeightsPerPlatform){
				var value = _currentWeightSet.round ? Math.round(text * _currentWeightSet.textInputMultiplyer) : text * _currentWeightSet.textInputMultiplyer;
				_textInputRight.text(this.formatWeightText(value))
				this.addWeightRight(this.createNewWeight(value));
				setTimeout($.proxy(this.resetRightTextInput, this), 1500);
			}
		}
		
		this.hideStartupInstructions();
	};
	
	p.resetRightTextInput = function(){
		_textInputRight.text('? '+_currentWeightSet.textInputLabel);		
	}
	
	p.totalWeightOnLeftSide = function (){
		var result = 0;
		for(var i=0, len=_skin.leftPlatform.weights.children.length; i < len; i++){
			var child = _skin.leftPlatform.weights.children[i];
			if(isDefined(child.weightValue)){
				result += child.weightValue;
			}
		}
		return result;
	}
	
	p.totalWeightOnRightSide = function(){
		var result = 0;
		for(var i=0, len=_skin.rightPlatform.weights.children.length; i < len; i++){
			var child = _skin.rightPlatform.weights.children[i];
			if(isDefined(child.weightValue)){
				result += child.weightValue;
			}
		}
		return result;
	}
	
	p.updateTotalWeights = function(){
		_skin.totalWeightSkin.totalWeightLeftTextField.text = this.formatWeightText(this.totalWeightOnLeftSide());
		_skin.totalWeightSkin.totalWeightRightTextField.text = this.formatWeightText(this.totalWeightOnRightSide());
	}
	
	p.updateScale = function(){
		var left = this.totalWeightOnLeftSide();
		var right = this.totalWeightOnRightSide();
		
		if(left == right){
			_skin.weigingBar.rotation = 0;
			_skin.leftPlatform.y = _platformBaseY;
			_skin.rightPlatform.y = _platformBaseY;
		} else if(left > right) {
			_skin.weigingBar.rotation = -_dr;
			_skin.leftPlatform.y = Math.round(_platformBaseY + _dy);
			_skin.rightPlatform.y = Math.round(_platformBaseY - _dy);
		} else {
			_skin.weigingBar.rotation = _dr;
			_skin.leftPlatform.y = Math.round(_platformBaseY - _dy);
			_skin.rightPlatform.y = Math.round(_platformBaseY + _dy);	
		}
	}
	
	p.addWeightLeft = function(weight){
		//Setup mouse things
		var onPressHandler = function(e){
			this.hideStartupInstructions();
							
			var target = e.target;
			var origX = target.x;
			var origY = target.y;
							
			var onMouseMoveHandler = function(e) {
				//Convert to local on the clock
				var local = _skin.leftPlatform.weights.globalToLocal(e.stageX, e.stageY);					
				target.x = local.x ;
				target.y = local.y ;
			}
			e.onMouseMove = $.proxy(onMouseMoveHandler,this);
			
			var onMouseUpHandler = function(e) {
				//Check left
				var dropLocal = _skin.leftPlatform.weightsContainerBackground.globalToLocal(e.stageX, e.stageY);
				if(_skin.leftPlatform.weightsContainerBackground.hitTest(dropLocal.x, dropLocal.y)){
					//Reset to original position	
					target.x = origX;
					target.y = origY;
					//Finished
					return;
				}
				
				//Remove the weight
				this.removeWeightLeft(target);
				
				//Check right
				var dropLocal = _skin.rightPlatform.weightsContainerBackground.globalToLocal(e.stageX, e.stageY);
				if(_skin.rightPlatform.weightsContainerBackground.hitTest(dropLocal.x, dropLocal.y) &&
				   _skin.rightPlatform.weights.getNumChildren() < _maxWeightsPerPlatform
				){
					this.addWeightRight(target);
				}
			}	
			
			//Set the mouseuphandler with proxy this
			e.onMouseUp = $.proxy(onMouseUpHandler,this);
		}
		//Set the onPress handler with a proxy on this so we can access this
		weight.onPress = $.proxy(onPressHandler, this);
		
		//Reorder container		
		_skin.leftPlatform.weights.addChild(weight);
		p.reoderContainer(_skin.leftPlatform.weights);
		
		//animate
		weight.gotoAndPlay('appear');
	};
	
	p.removeWeightLeft = function(weight){
		console.log('removing weight: '+weight);
		_skin.leftPlatform.weights.removeChild(weight);
		p.reoderContainer(_skin.leftPlatform.weights);
		
		//Remove mouse logic
		weight.onPress = null;
	};
	
	p.addWeightRight = function(weight){
		//Setup mouse things
		var onPressHandler = function(e){
			this.hideStartupInstructions();
							
			var target = e.target;
			var origX = target.x;
			var origY = target.y;
							
			var onMouseMoveHandler = function(e) {
				//Convert to local on the clock
				var local = _skin.rightPlatform.weights.globalToLocal(e.stageX, e.stageY);					
				target.x = local.x ;
				target.y = local.y ;
			}
			e.onMouseMove = $.proxy(onMouseMoveHandler,this);
			
			var onMouseUpHandler = function(e) {
				//Check right
				var dropLocal = _skin.rightPlatform.weightsContainerBackground.globalToLocal(e.stageX, e.stageY);
				if(_skin.rightPlatform.weightsContainerBackground.hitTest(dropLocal.x, dropLocal.y)){
					//Reset to original position	
					target.x = origX;
					target.y = origY;
					//Finished
					return;
				}
				
				//Remove the weight
				this.removeWeightRight(target);
				
				//Check right
				var dropLocal = _skin.leftPlatform.weightsContainerBackground.globalToLocal(e.stageX, e.stageY);
				if(_skin.leftPlatform.weightsContainerBackground.hitTest(dropLocal.x, dropLocal.y) &&
				   _skin.leftPlatform.weights.getNumChildren() < _maxWeightsPerPlatform
				){
					this.addWeightLeft(target);
				}
			}	
			
			//Set the mouseuphandler with proxy this
			e.onMouseUp = $.proxy(onMouseUpHandler,this);
		}
		//Set the onPress handler with a proxy on this so we can access this
		weight.onPress = $.proxy(onPressHandler, this);		
		//Reorder container
		_skin.rightPlatform.weights.addChild(weight);
		p.reoderContainer(_skin.rightPlatform.weights);
		
		//animate
		weight.gotoAndPlay('appear');
	}
	
	p.removeWeightRight = function(weight){
		//Remove skin
		_skin.rightPlatform.weights.removeChild(weight);
		p.reoderContainer(_skin.rightPlatform.weights);
	};
	
	p.reoderContainer = function(container){
		for(var i=0, len=container.children.length; i < len; i++){
			var child = container.children[i];
			child.x = Math.round(-118 + (i % 3) * 65); //-97
			child.y = Math.round(85 - Math.floor(i / 3) * 42); //
		}
	};
	
	p.initWeightBarWeights = function(){
		var weights = _currentWeightSet.availableWeights;
		
		for(var i=1; i < numWeights+1; i++){
			//Get the skin
			var weightSkin = _skin["weight"+i];
			
			if(i-1 < weights.length ){
				weightSkin.weightValue = weights[i-1];
				weightSkin.gotoAndStop('normal');
				weightSkin.weightTextField.text = this.formatWeightText(weightSkin.weightValue);	
				this.addWeightBarWeight(weightSkin);			
				
			} else {
				weightSkin.weightValue = 0;
				weightSkin.visible = false;
			}
		}
	}
	
	p.addWeightBarWeight = function(weight){
		
		//Setup mouse things
		//child.onPress 
		var onPressHandler = function(e){	
			
			//Hide instructions
			this.hideStartupInstructions();
			
			var target = e.target;
			var origX = target.x;
			var origY = target.y;
							
			var onMouseMoveHandler = function(e) {
				//Convert to local on the clock
				var local = _skin.globalToLocal(e.stageX, e.stageY);					
				target.x = local.x ;
				target.y = local.y ;
			}
			e.onMouseMove = $.proxy(onMouseMoveHandler,this);
			
			var onMouseUpHandler = function(e) {
				//Check left
				var dropLocal = _skin.leftPlatform.weightsContainerBackground.globalToLocal(e.stageX, e.stageY);
				if(_skin.leftPlatform.weightsContainerBackground.hitTest(dropLocal.x, dropLocal.y) &&
				   _skin.leftPlatform.weights.getNumChildren() < _maxWeightsPerPlatform
				){					
					var clone = this.createNewWeight(target.weightValue);
					this.addWeightLeft(clone);					
				}
				
				//Check right
				var dropLocal = _skin.rightPlatform.weightsContainerBackground.globalToLocal(e.stageX, e.stageY);
				if(_skin.rightPlatform.weightsContainerBackground.hitTest(dropLocal.x, dropLocal.y) &&
				   _skin.rightPlatform.weights.getNumChildren() < _maxWeightsPerPlatform
				){
					var clone = this.createNewWeight(target.weightValue);
					this.addWeightRight(clone);
				}
				
				//Reset to original position	
				target.x = origX;
				target.y = origY;						
			}	
			
			//Set the mouseuphandler with proxy this
			e.onMouseUp = $.proxy(onMouseUpHandler,this);
		}
		//Set the onPress handler with a proxy on this so we can access this
		weight.onPress = $.proxy(onPressHandler, this);
	}

	var _correctAnimationDisplayed = false;

	p.setNeutral = function(){
		//Reset animation
		_correctAnimationDisplayed = false;
	};
	
	p.setCorrect = function(){
		//Play animation		
		if(!_correctAnimationDisplayed){
			_skin.correctAnimation.gotoAndPlay(0);
			_correctAnimationDisplayed = true;
		}
	};
	
	p.setIncorrect = function(){
		//Reset animation
		_correctAnimationDisplayed = false;
	};
	
	p.tick = function(){
	
		//Update weights
		this.updateTotalWeights();
		this.updateScale();
		
		var left = this.totalWeightOnLeftSide();
		var right = this.totalWeightOnRightSide();
		
		if(left == right && left > 0){
			this.setCorrect();
		} else {
			this.setIncorrect();
		}		
	
		//Update the stage
		_stage.update();
	};
		
	p.reset = function(){
		_correctAnimationDisplayed = false;
		
		//Remove all weights
		_skin.leftPlatform.weights.removeAllChildren();
		_skin.rightPlatform.weights.removeAllChildren();
		
		//Reset textfields		
		this.resetLeftTextInput();
		this.resetRightTextInput();
	};
	
	p.close = function(){
		//Remove ourself from Notebook
		NB.getHostObject().deleteObject();		
	};
	
	p.save = function(){
		console.log("saving");
		var state = {
			"handwrittenInputLeft": _skin.textInputLeft.visible,
			"handwrittenInputRight": _skin.textInputRight.visible,
			"totalsLeft": _skin.totalWeightSkin.totalWeightLeftTextField.visible,
			"totalsRight": _skin.totalWeightSkin.totalWeightRightTextField.visible,
			"currentWeightSetName": _currentWeightSetName
		};
			
		//Add visibiblity fields
		for(var i=1; i <= numWeights; i++){			
			state["weight"+i] = _skin["weight"+i].visible;
		}
		
		//Serialize weightsLeft
		state.weightsLeft = [];
		for(var i=0, len=_skin.leftPlatform.weights.children.length; i < len; i++){
			var child = _skin.leftPlatform.weights.children[i];
			state.weightsLeft.push(child.weightValue);
		}

		//Serialize weightsRight
		state.weightsRight = [];
		for(var i=0, len=_skin.rightPlatform.weights.children.length; i < len; i++){
			var child = _skin.rightPlatform.weights.children[i];
			state.weightsRight.push(child.weightValue);
		}	
				
	
		//Now save the state at once
		NB.persist.save("state", state);
	};
	
	p.load = function(){
		var state = NB.persist.load("state");
		if(!isDefined(state))
			return;
		
		//Now parse it
		state = JSON.parse(state);
		this.currentWeightSetName = state.currentWeightSetName;
		_skin.textInputLeft.visible = state.handwrittenInputLeft;
		_skin.textInputRight.visible = state.handwrittenInputRight;
		_skin.totalWeightSkin.totalWeightLeftTextField.visible = state.totalsLeft;
		_skin.totalWeightSkin.totalWeightRightTextField.visible = state.totalsRight;	
		
		//Visibility of weights
		for(var i=1; i <= numWeights; i++){	
			if(i-1 < _currentWeightSet.availableWeights.length){
				_skin["weight"+i].visible = state["weight"+i];
			}		
		}
		//Add weights left
		for(var i=0, len=state.weightsLeft.length; i < len; i++){
			this.addWeightLeft(this.createNewWeight(state.weightsLeft[i]));
		}
		
		//Add weights right
		for(var i=0, len=state.weightsRight.length; i < len; i++){
			this.addWeightRight(this.createNewWeight(state.weightsRight[i]));
		}
		
		//Set correct to prevent animation
		_correctAnimationDisplayed = true;
	};
	
	p.openInfo = function(){
		//Switch view
		$('#frontend').hide(0, function(){
			$('#info').show(0);
		});	
	}
	
	p.closeInfo = function(){
		$('#info').hide(0, function(){
			$('#frontend').show(0);
		});
	}	
	
	p.openSettings = function(){		
		//Set the settings values
		$('#weightSetSelect').val(_currentWeightSetName);
		
		//Input fields
		$('input[name=displayLeftInput]').attr('checked', _skin.textInputLeft.visible);
		$('input[name=displayRightInput]').attr('checked', _skin.textInputRight.visible);
		//Totals
		$('input[name=displayLeftTotal]').attr('checked', _skin.totalWeightSkin.totalWeightLeftTextField.visible);
		$('input[name=displayRightTotal]').attr('checked', _skin.totalWeightSkin.totalWeightRightTextField.visible);
		
		this.updateWeightSelection();
		
		//Switch view
		$('#frontend').hide(0, function(){
			$('#backend').show(0);
		});		
	}
	
	p.makeAllWeightsVisible = function(){
		for(var i=1; i <= numWeights; i++){
			if(i-1 < _currentWeightSet.availableWeights.length){
				_skin["weight"+i].visible = true;
			} else {
				_skin["weight"+i].visible = false;
			}
		}		
	}
	
	p.updateWeightSelection = function (){
		for(var i=1; i <= numWeights; i++){
			$('input[name=weightCheckbox'+i+']').attr('checked', _skin["weight"+i].visible);
			
			if(i-1 < _currentWeightSet.availableWeights.length){
				$('#weightText'+i).text(this.formatWeightText(_currentWeightSet.availableWeights[i-1]));
				$('#weightDiv'+i).show();
			} else {
				//Make div invisible
				$('#weightText'+i).text("");
				$('#weightDiv'+i).hide();
			}
		}		
	}
	
	p.closeSettings = function(){	
		//Read settings values		
		_skin.textInputLeft.visible = $('input[name=displayLeftInput]').is(':checked');
		_skin.textInputRight.visible = $('input[name=displayRightInput]').is(':checked');
		_skin.totalWeightSkin.totalWeightLeftTextField.visible = $('input[name=displayLeftTotal]').is(':checked');
		_skin.totalWeightSkin.totalWeightRightTextField.visible = $('input[name=displayRightTotal]').is(':checked');
			
		for(var i=1; i <= numWeights; i++){			
			_skin["weight"+i].visible = $('input[name=weightCheckbox'+i+']').is(':checked');
		}
		
		//force an update of the stage to make sure the graphic has updated before the user sees it
		_stage.update();
		
		//Save state
		this.save();
		
		//Switch view
		$('#backend').hide(0, function(){
			$('#frontend').show(0);	
		});		
	}
	
	p.formatWeightText = function(value){
		if(_currentWeightSet.format == "metric"){
			return this.formatWeightTextMetric(value);
		} else {
			return this.formatWeightTextCustomary(value);
		}
		return ""+value;	
	}
	
	p.formatWeightTextMetric = function(value){
		if(value >= 1000000){
			return NumberUtils.round(value/1000000, 3)+" kg";
		}else if(value >= 1000){
			return NumberUtils.round(value/1000, 3)+" g";
		} 
		return value+" mg";		
	}
	
	p.formatWeightTextCustomary = function(value){
		var pounds = Math.floor(value / 16);
		var ounces = value % 16;
		
		var result = "";
		if(pounds > 0){
			result += pounds+"lb ";
		}
		if(ounces > 0 || pounds == 0){
			result += ounces+"oz";
		}
		return result;		
	}
	
	p.inject = function(){
		var host = NB.getHostObject();
		var center = new Point(host.x + host.width/2, host.y + host.height/2);	
		
		//Left	
		if(_skin.totalWeightSkin.totalWeightLeftTextField.visible){
			var text = this.formatWeightText(this.totalWeightOnLeftSide());			
			var prototype = NB.objectPrototype.text(text, { x: host.x + 50 , 
															y: center.y - 100 , 
															height: 150,
															width: text.length * 30, //For some reason font size is not enough
															transparency:0});
			prototype.font.name = "Comic Sans MS"
			var object = NB.addObject(prototype);
			object.animation.start({ speed: "Normal", repeats: "None", type: "Fade In" });	
		}
		//Right
		if(_skin.totalWeightSkin.totalWeightRightTextField.visible){
			var text = this.formatWeightText(this.totalWeightOnRightSide());
			var prototype = NB.objectPrototype.text(text, { x: center.x + 50, 
															y: center.y - 100, 
															height: 150,
															width: text.length * 30, //For some reason font size is not enough
															transparency:0});
			prototype.font.name = "Comic Sans MS"
			var object = NB.addObject(prototype);
			object.animation.start({ speed: "Normal", repeats: "None", type: "Fade In" });
		}		
	}
	window.WeigingScaleWidget = WeigingScaleWidget;
})(window, jQuery);